@@ -1,5 +1,3 @@
-// -*- C++ -*-
-
 /* main.cc
  *
  * Copyright (C) 2002 The libxml++ development team
@@ -19,14 +17,13 @@
  * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
 
 #include <libxml++/libxml++.h>
 #include <iostream>
-#include <stdlib.h>
+#include <cstdlib>
 
 int main(int argc, char* argv[])
 {
@@ -34,43 +31,57 @@ int main(int argc, char* argv[])
   // so we can use std::cout with UTF-8, via Glib::ustring, without exceptions.
   std::locale::global(std::locale(""));
 
-  std::string schemafilepath("example.xsd"),
-              docfilepath("example.xml");
+  std::string docfilepath("example.xml");
+  std::string xsdschemafilepath("example.xsd");
+  std::string rngschemafilepath("example.rng");
 
-  if(argc!=1 && argc!=3)
+  if (argc != 1 && argc != 4)
   {
-    std::cout << "usage : " << argv[0] << " [document schema]" << std::endl;
+    std::cout << "usage : " << argv[0] << " [document schema.xsd schema.rng]" << std::endl;
     return EXIT_FAILURE;
   }
 
-  if(argc == 3)
+  if (argc == 4)
   {
     docfilepath = argv[1];
-    schemafilepath = argv[2];
+    xsdschemafilepath = argv[2];
+    rngschemafilepath = argv[3];
   }
 
+  Glib::ustring phase;
   try
   {
-    xmlpp::DomParser       parser(docfilepath);
-    xmlpp::SchemaValidator validator(schemafilepath);
+    phase = "XSD";
+    xmlpp::XsdValidator xsdvalidator(xsdschemafilepath);
+
+    phase = "RelaxNG";
+    xmlpp::RelaxNGValidator rngvalidator(rngschemafilepath);
 
     try
     {
-      validator.validate( parser.get_document() );
-      std::cout << "Valid document" << std::endl;
+      phase = "parsing";
+      xmlpp::DomParser parser(docfilepath);
+
+      phase = "XSD validating";
+      xsdvalidator.validate(parser.get_document());
+      std::cout << "Valid document, XsdValidator" << std::endl;
+
+      phase = "RelaxNG validating";
+      rngvalidator.validate(parser.get_document());
+      std::cout << "Valid document, RelaxNGValidator" << std::endl;
     }
-    catch( const xmlpp::validity_error& error)
+    catch (const xmlpp::exception& ex)
     {
-      std::cerr << "Error validating the document" << std::endl;
-      std::cerr << error.what();
+      std::cerr << "Error " << phase << " the document" << std::endl;
+      std::cerr << ex.what() << std::endl;
       return EXIT_FAILURE;
     }
   }
-  catch( const xmlpp::parse_error& error)
+  catch (const xmlpp::exception& ex)
   {
-    std::cerr << "Error parsing the schema: " << error.what() << std::endl;
+    std::cerr << "Error parsing the " << phase << " schema" << std::endl;
+    std::cerr << ex.what() << std::endl;
     return EXIT_FAILURE;
   }
   return EXIT_SUCCESS;
 }
-
