@@ -25,14 +25,14 @@ Glib::ustring AttributeDeclaration::get_value() const
   return (const char*)cobj()->defaultValue;
 }
 
-xmlAttribute* AttributeDeclaration::cobj()
+xmlAttribute* AttributeDeclaration::cobj() noexcept
 {
   // An XML_ATTRIBUTE_DECL is represented by an xmlAttribute struct. Reinterpret
   // the xmlNode pointer stored in the base class as an xmlAttribute pointer.
   return reinterpret_cast<xmlAttribute*>(Node::cobj());
 }
 
-const xmlAttribute* AttributeDeclaration::cobj() const
+const xmlAttribute* AttributeDeclaration::cobj() const noexcept
 {
   // An XML_ATTRIBUTE_DECL is represented by an xmlAttribute struct. Reinterpret
   // the xmlNode pointer stored in the base class as an xmlAttribute pointer.
